'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deepRemove = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _immutable = require('immutable');

var _immutabilityHelper = require('immutability-helper');

var _immutabilityHelper2 = _interopRequireDefault(_immutabilityHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var deepRemove = exports.deepRemove = function deepRemove(map, id) {
  var children = map.getItem(id).children;
  if (children) children.forEach(function (child) {
    map = deepRemove(map, child);
  });
  return map.deleteIn(['items', id]);
};

var createItems = function createItems(type) {
  return (0, _immutable.Map)({
    root: { id: 'root', type: type, props: {}, children: [] }
  });
};

var defaultItems = (0, _immutable.Map)({ root: {
    id: 'root',
    type: 'Column',
    props: {},
    children: [],
    style: {}
  } });

var validateShape = function validateShape(items) {
  if ((typeof items === 'undefined' ? 'undefined' : _typeof(items)) !== 'object' || Array.isArray(items)) return false;
  if (!items['root']) return false;
  Object.keys(items).forEach(function (id) {
    var item = items[id];
    if (!(item.id && item.type && item.props)) return false;
  });
  return true;
};

var LayoutState = function (_Record) {
  _inherits(LayoutState, _Record);

  function LayoutState(type) {
    _classCallCheck(this, LayoutState);

    if (!(typeof type === 'string' || validateShape(type))) {
      throw new Error('LayoutState must be supplied either a root Component type or items data');
    }
    if (type instanceof Object) {
      var _this = _possibleConstructorReturn(this, (LayoutState.__proto__ || Object.getPrototypeOf(LayoutState)).call(this, { items: (0, _immutable.Map)(type) }));
    } else {
      var _this = _possibleConstructorReturn(this, (LayoutState.__proto__ || Object.getPrototypeOf(LayoutState)).call(this, { items: createItems(type) }));
    }
    return _possibleConstructorReturn(_this);
  }

  /**
   * Gets an item by id
   * @param {Number} id
   * @return {Object} item
   */


  _createClass(LayoutState, [{
    key: 'getItem',
    value: function getItem(id) {
      return this.items.get(id);
      // const item = this.items.get(id);
      // return item && item.toJS();
    }
  }, {
    key: 'insertOrMoveItem',
    value: function insertOrMoveItem(parentId, idx, item) {
      return item.id ? this.moveItem(parentId, idx, item) : this.insertItem(parentId, idx, item);
    }
  }, {
    key: 'insertItem',
    value: function insertItem(parentId, idx, item) {
      item.id = this.generateRandomKey();
      item.parent = parentId;
      var nextState = this.setIn(['items', item.id], item).updateItem(parentId, { children: { $splice: [[idx, 0, item.id]] } });
      return nextState;
    }
  }, {
    key: 'moveItem',
    value: function moveItem(parentId, idx, item) {
      var nextState = this.updateItem(item.parent, { children: { $apply: function $apply(c) {
            return c.filter(function (id) {
              return id !== item.id;
            });
          } } }).updateItem(parentId, { children: { $splice: [[idx, 0, item.id]] } }).updateItem(item.id, { parent: { $set: parentId } });
      return nextState;
    }
  }, {
    key: 'updateItem',
    value: function updateItem(id, updater) {
      return this.updateIn(['items', id], function (item) {
        return (0, _immutabilityHelper2.default)(item, updater);
      });
    }
  }, {
    key: 'removeItem',
    value: function removeItem(id) {
      if (id === 'root') return this;
      var item = this.getItem(id);
      var parentId = item.parent;
      var nextState = this.updateItem(parentId, { children: { $apply: function $apply(c) {
            return c.filter(function (cId) {
              return cId !== id;
            });
          } } });
      return deepRemove(nextState, id);
    }
  }, {
    key: 'setSelectedItem',
    value: function setSelectedItem(id) {
      if (this.selectedItem === id) return;
      return this.set('selectedItem', id);
    }
  }, {
    key: 'getSelectedItem',
    value: function getSelectedItem() {
      return this.items.get(this.selectedItem);
    }
  }, {
    key: 'getAncestors',
    value: function getAncestors(id) {
      var result = [this.getItem(id)];
      while (result[0].parent && result.length < 4) {
        result.unshift(this.getItem(result[0].parent));
      }
      return result;
    }
  }, {
    key: 'toRaw',
    value: function toRaw() {
      return this.items.toJS();
    }
  }, {
    key: 'generateRandomKey',
    value: function generateRandomKey() {
      var key = void 0;
      while (key === undefined || this.items.has(key) || !isNaN(Number(key))) {
        key = Math.floor(Math.random() * Math.pow(2, 24)).toString(32);
      }
      return key;
    }
  }]);

  return LayoutState;
}((0, _immutable.Record)({ items: (0, _immutable.Map)(), selectedItem: null }));

exports.default = LayoutState;