'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _LayoutState = require('../../model/LayoutState');

var _LayoutState2 = _interopRequireDefault(_LayoutState);

var _immutable = require('immutable');

var _immutabilityHelper = require('immutability-helper');

var _immutabilityHelper2 = _interopRequireDefault(_immutabilityHelper);

var _actions = require('./actions');

var ACTIONS = _interopRequireWildcard(_actions);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Reducer = function Reducer(layoutstate) {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : layoutstate;
    var action = arguments[1];

    var nextState = void 0;
    switch (action.type) {
      case ACTIONS.SET_LAYOUT_STATE:
        return action.layoutState;
      case ACTIONS.INSERT_OR_MOVE_ITEM:
        if (action.item.id) {
          // Item already exists, MOVE_ITEM
          var oldParent = state.getItem(action.item.id).parent;
          action.item.parent = action.parentId;
          nextState = state.updateIn(['items', oldParent], function (item) {
            return (0, _immutabilityHelper2.default)(item, {
              children: { $apply: function $apply(c) {
                  return c.filter(function (cId) {
                    return cId !== action.item.id;
                  });
                } }
            });
          }).updateIn(['items', action.parentId], function (item) {
            return (0, _immutabilityHelper2.default)(item, {
              children: { $splice: [[action.idx, 0, action.item.id]] }
            });
          });
        } else {
          // Item doesnt exist, INSERT_ITEM
          action.item.id = generateRandomKey(state.items);
          action.item.parent = action.parentId;
          nextState = state.setIn(['items', action.item.id], action.item).updateIn(['items', action.parentId], function (item) {
            return (0, _immutabilityHelper2.default)(item, {
              children: { $splice: [[action.idx, 0, action.item.id]] }
            });
          });
        }
        return nextState;
      case ACTIONS.UPDATE_ITEM:
        return state.updateIn(['items', action.id], function (item) {
          return (0, _immutabilityHelper2.default)(item, action.updater);
        });
      case ACTIONS.REMOVE_ITEM:
        if (action.id === 'root') return state;
        var children = [action.id];
        var parentRef = state.getItem(action.id).parent;
        nextState = state.updateIn(['items', parentRef], function (item) {
          return (0, _immutabilityHelper2.default)(item, {
            children: { $apply: function $apply(children) {
                return children.filter(function (cId) {
                  return cId !== action.id;
                });
              } }
          });
        });
        while (children.length) {
          var id = children.pop();
          var item = state.getIn(['items', id]);
          children = children.concat(item.children);
          nextState = nextState.deleteIn(['items', id]);
        }
        return nextState;
      default:
        return state;
    }
  };
};

var generateRandomKey = function generateRandomKey(items) {
  var key = void 0;
  while (key === undefined || items.has(key) || !isNaN(Number(key))) {
    key = Math.floor(Math.random() * Math.pow(2, 24)).toString(32);
  }
  return key;
};

exports.default = Reducer;
module.exports = exports['default'];