'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _InnerWrapper = require('../components/InnerWrapper');

var _InnerWrapper2 = _interopRequireDefault(_InnerWrapper);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var processPlugins = function processPlugins(props) {
  var RootWrapper = _InnerWrapper2.default;
  var RootProvider = function RootProvider(_ref) {
    var children = _ref.children;
    return _react2.default.Children.only(children);
  };
  var reducers = {};
  var middlewares = [];
  var plugins = [];
  var names = new Set();
  props.plugins.forEach(function (pluginFactory, idx) {
    if (typeof pluginFactory !== 'function') {
      console.error('Error at plugins[' + idx + ']: Expected plugin to be a function, instead got a ' + (typeof pluginFactory === 'undefined' ? 'undefined' : _typeof(pluginFactory)));
      return;
    }
    var plugin = pluginFactory(props);
    plugins.push(plugin);
    if (names.has(plugin.Name)) {
      console.error('Plugin name conflict. More than one plugin with the name "' + plugin.Name + '". Ensure you are not using multiple copies of one plugin.');
    } else {
      names.add(plugin.Name);
    }
    if (plugin.Wrapper) RootWrapper = plugin.Wrapper(RootWrapper);
    if (plugin.Provider) RootProvider = plugin.Provider(RootProvider);
    if (plugin.reducer) reducers[plugin.Name] = plugin.reducer;
    if (plugin.middleware) middlewares.push(plugin.middleware);
  });
  return {
    RootWrapper: RootWrapper,
    RootProvider: RootProvider,
    reducers: reducers,
    middlewares: middlewares,
    plugins: plugins
  };
};

exports.default = processPlugins;
module.exports = exports['default'];